/* add dollar sign in console code-block */
div.highlight-console pre span.go::before {
  content: "$";
  margin-right: 10px;
  margin-left: 5px;
}

/* custom colors for titles */
html[data-theme="light"] {
  --pst-color-primary: #4b9cd3;
}

html[data-theme="dark"] {
  --pst-color-primary: #4b9cd3;
}
